@@ -147,9 +147,12 @@ class EC2SubnetRouteTableAssociation(GenericBaseModel):
         gw_id = self.resolve_refs_recursively(stack_name, props.get("GatewayId"), resources)
         route_tables = client.describe_route_tables()["RouteTables"]
         route_table = ([t for t in route_tables if t["RouteTableId"] == table_id] or [None])[0]
+        subnet_id = self.resolve_refs_recursively(stack_name, props.get("SubnetId"), resources)
         if route_table:
             associations = route_table.get("Associations", [])
             association = [a for a in associations if a.get("GatewayId") == gw_id]
+            if subnet_id:
+                association = [a for a in associations if a.get("SubnetId") == subnet_id]
             return (association or [None])[0]
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
@@ -165,7 +168,11 @@ class EC2SubnetRouteTableAssociation(GenericBaseModel):
                     "RouteTableId": "RouteTableId",
                     "SubnetId": "SubnetId",
                 },
-            }
+            },
+            "delete": {
+                "function": "disassociate_route_table",
+                "parameters": {"AssociationId": "RouteTableAssociationId"},
+            },
         }
 
 
@@ -331,6 +338,10 @@ class EC2VPC(GenericBaseModel):
                     ]
                 )
                 for rt in resp["RouteTables"]:
+                    for assoc in rt.get("Associations", []):
+                        ec2_client.disassociate_route_table(
+                            AssociationId=assoc["RouteTableAssociationId"]
+                        )
                     ec2_client.delete_route_table(RouteTableId=rt["RouteTableId"])
 
         return {
